<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Model;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Sylius\Component\Core\Model\ImageInterface;
use Sylius\Component\Resource\Model\TimestampableTrait;
use Sylius\Component\Resource\Model\ToggleableTrait;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;
use Symfony\Component\Serializer\Attribute\Groups;
use Symfony\Component\Serializer\Attribute\SerializedName;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * @UniqueEntity("code", groups={"sylius", "default"})
 */
class VoucherConfiguration implements VoucherConfigurationInterface
{
    use TimestampableTrait;
    use ToggleableTrait;

    protected ?int $id;

    protected ?string $code;

    /** @var ImageInterface[]|Collection */
    protected Collection $images;

    /**
     * @var VoucherChannelConfigurationInterface[]|Collection
     *
     * @Assert\Valid(groups={"sylius", "default"})
     */
    protected Collection $channelConfigurations;

    protected bool $default = false;

    public function __construct()
    {
        $this->images = new ArrayCollection();
        $this->channelConfigurations = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getCode(): ?string
    {
        return $this->code;
    }

    public function setCode(?string $code): void
    {
        $this->code = $code;
    }

    public function getImages(): Collection
    {
        return $this->images;
    }

    public function getImagesByType(string $type): Collection
    {
        return $this->images->filter(fn (ImageInterface $image): bool => $image->getType() === $type);
    }

    public function hasImages(): bool
    {
        return !$this->getImages()->isEmpty();
    }

    public function hasImage(ImageInterface $image): bool
    {
        return $this->getImages()->contains($image);
    }

    public function addImage(ImageInterface $image): void
    {
        if (!$this->hasImage($image)) {
            $image->setOwner($this);
            $this->images->add($image);
        }
    }

    public function removeImage(ImageInterface $image): void
    {
        if ($this->hasImage($image)) {
            $image->setOwner(null);
            $this->images->removeElement($image);
        }
    }

    /**
     * @SerializedName("backgroundImage")
     *
     * @Groups({"admin:voucher-configuration:read"})
     */
    public function getBackgroundImage(): ?VoucherConfigurationImageInterface
    {
        $images = $this->getImagesByType(VoucherConfigurationImageInterface::TYPE_BACKGROUND);
        if ($images->isEmpty()) {
            return null;
        }

        /** @var VoucherConfigurationImageInterface $image */
        $image = $images->first();

        return $image;
    }

    public function setBackgroundImage(?VoucherConfigurationImageInterface $image): void
    {
        $actualImage = $this->getBackgroundImage();
        if (null !== $actualImage) {
            $this->removeImage($actualImage);
        }

        if (null === $image) {
            return;
        }

        if (VoucherConfigurationImageInterface::TYPE_BACKGROUND !== $image->getType()) {
            $image->setType(VoucherConfigurationImageInterface::TYPE_BACKGROUND);
        }
        $this->addImage($image);
    }

    public function getChannelConfigurations(): Collection
    {
        return $this->channelConfigurations;
    }

    public function hasChannelConfigurations(): bool
    {
        return !$this->channelConfigurations->isEmpty();
    }

    public function hasChannelConfiguration(VoucherChannelConfigurationInterface $channelConfiguration): bool
    {
        return $this->channelConfigurations->contains($channelConfiguration);
    }

    public function addChannelConfiguration(VoucherChannelConfigurationInterface $channelConfiguration): void
    {
        if (!$this->hasChannelConfiguration($channelConfiguration)) {
            $channelConfiguration->setConfiguration($this);
            $this->channelConfigurations->add($channelConfiguration);
        }
    }

    public function removeChannelConfiguration(VoucherChannelConfigurationInterface $channelConfiguration): void
    {
        if ($this->hasChannelConfiguration($channelConfiguration)) {
            $channelConfiguration->setConfiguration(null);
            $this->channelConfigurations->removeElement($channelConfiguration);
        }
    }

    public function isDefault(): bool
    {
        return $this->default;
    }

    public function setDefault(bool $default): void
    {
        $this->default = $default;
    }
}
